@@ -2,8 +2,10 @@ use crate::{
     error::{GenericError, IndexChangeError, PERes, PIRes, ReadError},
     id::{index_id_to_segment_id_data, index_id_to_segment_id_meta, IndexId, SegmentId},
     index::{
-        config::{ByteVec, IndexOrd, IndexType, Indexes, ValueMode, WrapperType},
+        bytevec::ByteVec,
+        config::{IndexOrd, IndexTypeInternal, Indexes, ValueMode},
         serialization::{deserialize, serialize},
+        string_wrapper::StringWrapper,
         tree::{
             nodes::{compare, Node, NodeRef, PageIter, PageIterBack, Value},
             IndexApply, KeyChanges, ValueChange as TreeValue,
@@ -24,41 +26,6 @@ use std::{
     vec::IntoIter,
 };
 
-#[derive(Clone)]
-pub struct Container<T: IndexOrd> {
-    value: T,
-}
-impl<T> WrapperType<T> for Container<T>
-where
-    T: IndexType,
-    Self: Extractor,
-{
-    fn value(self) -> T {
-        self.value
-    }
-}
-impl<T: IndexOrd> PartialOrd for Container<T> {
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        Some(IndexOrd::cmp(&self.value, &other.value))
-    }
-}
-impl<T: IndexOrd> Ord for Container<T> {
-    fn cmp(&self, other: &Self) -> Ordering {
-        IndexOrd::cmp(&self.value, &other.value)
-    }
-}
-impl<T: IndexOrd> PartialEq for Container<T> {
-    fn eq(&self, other: &Self) -> bool {
-        IndexOrd::cmp(&self.value, &other.value) == Ordering::Equal
-    }
-}
-impl<T: IndexOrd> Eq for Container<T> {}
-impl<T: IndexOrd> From<T> for Container<T> {
-    fn from(value: T) -> Self {
-        Container { value }
-    }
-}
-
 #[derive(Clone, Debug, PartialEq)]
 pub enum ValueChange<V> {
     Add(V),
@@ -85,68 +52,91 @@ impl Changes {
     }
 }
 
-fn add_value<V: IndexType>(values: &mut ValueContainer, val: V) -> usize {
-    let v = V::Wrapper::get_vec_mut(values);
+fn add_value<V: Extractor>(values: &mut ValueContainer, val: V) -> usize {
+    let v = V::get_vec_mut(values);
     let l = v.len();
-    v.push(val.into());
+    v.push(val);
     l
 }
-fn resolve_values<V: IndexType>(values: &ValueContainer, changes: Changes) -> Vec<ValueChange<V>> {
-    let v = V::Wrapper::get_vec(values);
+fn resolve_values<V: Extractor>(values: &ValueContainer, changes: Changes) -> Vec<ValueChange<V>> {
+    let v = V::get_vec(values);
     changes
         .changes
         .iter()
         .map(|c| match c {
-            Change::Add(p) => ValueChange::Add(v[*p].clone().value()),
-            Change::Remove(o) => ValueChange::Remove(o.map(|p| v[p].clone().value())),
+            Change::Add(p) => ValueChange::Add(v[*p].clone()),
+            Change::Remove(o) => ValueChange::Remove(o.map(|p| v[p].clone())),
         })
         .collect()
 }
-fn add_entry<K: IndexType>(entries: &mut EntriesContainer, k: K, change: Change) {
-    let v = K::Wrapper::get_entries_mut(entries).expect("wrong match from the type and the value container");
-    match v.entry(k.into()) {
-        BTreeEntry::Occupied(ref mut o) => {
-            o.get_mut().push(change);
+fn add_entry<K: Extractor>(entries: &mut EntriesContainer, k: K, change: Change) {
+    let v = K::get_entries_mut(entries).expect("wrong match from the type and the value container");
+    match v.binary_search_by(|n| compare(&n.0, &k)) {
+        Ok(index) => {
+            v[index].1.push(change);
         }
-        BTreeEntry::Vacant(va) => {
-            va.insert(Changes::new(change));
+        Err(index) => {
+            v.insert(index, (k, Changes::new(change)));
         }
     }
 }
 
-fn get_changes<K: IndexType>(entries: &EntriesContainer, x: &K) -> Option<Changes> {
-    if let Some(v) = K::Wrapper::get_entries(entries) {
-        let k = x.clone().into();
-        v.get(&k).map(Clone::clone)
+fn get_changes<K: Extractor>(entries: &EntriesContainer, k: &K) -> Option<Changes> {
+    if let Some(v) = K::get_entries(entries) {
+        match v.binary_search_by(|n| compare(&n.0, k)) {
+            Ok(index) => v.get(index).map(|x| x.1.clone()),
+            Err(_) => None,
+        }
     } else {
         None
     }
 }
 
-fn resolve_range<T: IndexType, R>(entries: &EntriesContainer, range: R) -> IntoIter<T>
+fn resolve_range<T: Extractor, R>(entries: &EntriesContainer, range: R) -> IntoIter<T>
 where
     R: RangeBounds<T>,
 {
-    let into_range: (Bound<T::Wrapper>, Bound<T::Wrapper>) =
-        (map_bound(range.start_bound()), map_bound(range.end_bound()));
-    let v = T::Wrapper::get_entries(entries).expect("wrong match from the type and the value container");
-    BTreeMap::range(v, into_range)
-        .map(|x| x.0.clone().value())
-        .collect::<Vec<T>>()
+    let v = T::get_entries(entries).expect("wrong match from the type and the value container");
+    let index = match range.start_bound() {
+        Bound::Included(k) => match v.binary_search_by(|n| compare(&n.0, k)) {
+            Ok(index) => index,
+            Err(index) => index,
+        },
+        Bound::Excluded(k) => match v.binary_search_by(|n| compare(&n.0, k)) {
+            Ok(index) => index + 1,
+            Err(index) => index,
+        },
+        Bound::Unbounded => 0,
+    };
+    let end_index = match range.end_bound() {
+        Bound::Included(k) => match v.binary_search_by(|n| compare(&n.0, k)) {
+            Ok(index) => index,
+            Err(index) => index - 1,
+        },
+        Bound::Excluded(k) => match v.binary_search_by(|n| compare(&n.0, k)) {
+            Ok(index) => index - 1,
+            Err(index) => index - 1,
+        },
+        Bound::Unbounded => v.len() - 1,
+    };
+    v[index..=end_index]
+        .iter()
+        .map(|(val, _): &(T, _)| val.clone())
+        .collect::<Vec<_>>()
         .into_iter()
 }
 
 macro_rules! impl_index_data_type {
     ($t:ty, $v:path, $v2:path) => {
-        impl Extractor for <$t as IndexType>::Wrapper {
-            fn get_vec_mut(vc: &mut ValueContainer) -> &mut Vec<Self> {
+        impl Extractor for $t {
+            fn get_vec_mut(vc: &mut ValueContainer) -> &mut Vec<$t> {
                 if let $v(ref mut v) = vc {
                     v
                 } else {
                     panic!("wrong match from type and value container")
                 }
             }
-            fn get_vec(vc: &ValueContainer) -> &Vec<Self> {
+            fn get_vec(vc: &ValueContainer) -> &Vec<$t> {
                 if let $v(ref v) = vc {
                     v
                 } else {
@@ -154,14 +144,14 @@ macro_rules! impl_index_data_type {
                 }
             }
 
-            fn get_entries(vc: &EntriesContainer) -> Option<&BTreeMap<Self, Changes>> {
+            fn get_entries(vc: &EntriesContainer) -> Option<&Vec<($t, Changes)>> {
                 if let $v2(ref v) = vc {
                     Some(v)
                 } else {
                     None
                 }
             }
-            fn get_entries_mut(vc: &mut EntriesContainer) -> Option<&mut BTreeMap<Self, Changes>> {
+            fn get_entries_mut(vc: &mut EntriesContainer) -> Option<&mut Vec<($t, Changes)>> {
                 if let $v2(ref mut v) = vc {
                     Some(v)
                 } else {
@@ -169,7 +159,7 @@ macro_rules! impl_index_data_type {
                 }
             }
             fn new_entries() -> EntriesContainer {
-                $v2(BTreeMap::new())
+                $v2(Vec::new())
             }
 
             fn new_values() -> ValueContainer {
@@ -184,14 +174,14 @@ macro_rules! container_enums {
         #[derive(Clone)]
         pub enum EntriesContainer {
             $(
-            $variant(BTreeMap<<$t as IndexType>::Wrapper, Changes>),
+            $variant(Vec<($t, Changes)>),
             )+
         }
 
         #[derive(Clone)]
         pub enum ValueContainer {
             $(
-            $variant(Vec<<$t as IndexType>::Wrapper>),
+            $variant(Vec<$t>),
             )+
         }
 
@@ -211,13 +201,13 @@ macro_rules! container_enums {
 
         fn valapplier<K>(
             values: &ValueContainer,
-            k: &BTreeMap<K::Wrapper, Changes>,
+            k: &[(K, Changes)],
             index_id: &IndexId,
             persy: &PersyImpl,
             tx: &mut TransactionImpl,
         ) -> PIRes<()>
         where
-            K: IndexType,
+            K: IndexTypeInternal,
         {
             match values {
                 $(
@@ -245,16 +235,16 @@ container_enums!(
     I128<i128>,
     F32W<f32>,
     F64W<f64>,
-    String<String>,
+    StringWrapper<StringWrapper>,
     PersyId<PersyId>,
     ByteVec<ByteVec>,
 );
 
-pub trait Extractor: Sized {
+pub trait Extractor: IndexOrd + Sized + Clone {
     fn get_vec_mut(vc: &mut ValueContainer) -> &mut Vec<Self>;
     fn get_vec(vc: &ValueContainer) -> &Vec<Self>;
-    fn get_entries(vc: &EntriesContainer) -> Option<&BTreeMap<Self, Changes>>;
-    fn get_entries_mut(vc: &mut EntriesContainer) -> Option<&mut BTreeMap<Self, Changes>>;
+    fn get_entries(vc: &EntriesContainer) -> Option<&Vec<(Self, Changes)>>;
+    fn get_entries_mut(vc: &mut EntriesContainer) -> Option<&mut Vec<(Self, Changes)>>;
     fn new_entries() -> EntriesContainer;
     fn new_values() -> ValueContainer;
 }
@@ -263,12 +253,12 @@ fn apply_to_index<K, V>(
     persy: &PersyImpl,
     tx: &mut TransactionImpl,
     index_id: &IndexId,
-    keys: &BTreeMap<K::Wrapper, Changes>,
-    values: &[V::Wrapper],
+    keys: &[(K, Changes)],
+    values: &[V],
 ) -> PIRes<()>
 where
-    K: IndexType,
-    V: IndexType,
+    K: IndexTypeInternal,
+    V: IndexTypeInternal,
 {
     let changes: Vec<_> = keys
         .iter()
@@ -277,11 +267,11 @@ where
                 .changes
                 .iter()
                 .map(|ch| match *ch {
-                    Change::Add(pos) => TreeValue::Add(values[pos].clone().value()),
-                    Change::Remove(pos) => TreeValue::Remove(pos.map(|p| values[p].clone().value())),
+                    Change::Add(pos) => TreeValue::Add(values[pos].clone()),
+                    Change::Remove(pos) => TreeValue::Remove(pos.map(|p| values[p].clone())),
                 })
                 .collect();
-            KeyChanges::new(k.clone().value(), vals)
+            KeyChanges::new(k.clone(), vals)
         })
         .collect();
     let mut index = Indexes::get_index_keeper_tx::<K, V>(persy, tx, index_id)?;
@@ -290,14 +280,6 @@ where
     Ok(())
 }
 
-fn map_bound<T: Into<T1> + Clone, T1>(b: Bound<&T>) -> Bound<T1> {
-    match b {
-        Bound::Excluded(x) => Bound::Excluded((*x).clone().into()),
-        Bound::Included(x) => Bound::Included((*x).clone().into()),
-        Bound::Unbounded => Bound::Unbounded,
-    }
-}
-
 pub struct IndexTransactionKeeper {
     indexex_changes: BTreeMap<IndexId, (EntriesContainer, ValueContainer)>,
 }
@@ -311,8 +293,8 @@ impl IndexTransactionKeeper {
 
     pub fn put<K, V>(&mut self, index: IndexId, k: K, v: V)
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         match self.indexex_changes.entry(index) {
             BTreeEntry::Occupied(ref mut o) => {
@@ -321,8 +303,8 @@ impl IndexTransactionKeeper {
                 add_entry(entries, k, Change::Add(pos));
             }
             BTreeEntry::Vacant(va) => {
-                let mut values = V::Wrapper::new_values();
-                let mut keys = K::Wrapper::new_entries();
+                let mut values = V::new_values();
+                let mut keys = K::new_entries();
                 let pos = add_value(&mut values, v);
                 add_entry(&mut keys, k, Change::Add(pos));
                 va.insert((keys, values));
@@ -332,8 +314,8 @@ impl IndexTransactionKeeper {
 
     pub fn remove<K, V>(&mut self, index: IndexId, k: K, v: Option<V>)
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         match self.indexex_changes.entry(index) {
             BTreeEntry::Occupied(ref mut o) => {
@@ -341,8 +323,8 @@ impl IndexTransactionKeeper {
                 add_entry(&mut o.get_mut().0, k, Change::Remove(pos));
             }
             BTreeEntry::Vacant(va) => {
-                let mut values = V::Wrapper::new_values();
-                let mut keys = K::Wrapper::new_entries();
+                let mut values = V::new_values();
+                let mut keys = K::new_entries();
                 let pos = v.map(|val| add_value(&mut values, val));
                 add_entry(&mut keys, k, Change::Remove(pos));
                 va.insert((keys, values));
@@ -352,8 +334,8 @@ impl IndexTransactionKeeper {
 
     pub fn get_changes<K, V>(&self, index: IndexId, k: &K) -> Option<Vec<ValueChange<V>>>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         self.indexex_changes
             .get(&index)
@@ -364,13 +346,14 @@ impl IndexTransactionKeeper {
     pub fn apply_changes<K, V>(
         &self,
         index_id: IndexId,
+        index_name: &str,
         vm: ValueMode,
         k: &K,
         pers: Option<Value<V>>,
     ) -> Result<Option<Value<V>>, IndexChangeError>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
     {
         let mut result = pers;
         if let Some(key_changes) = self.get_changes::<K, V>(index_id, k) {
@@ -384,9 +367,8 @@ impl IndexTransactionKeeper {
                                     if compare(&v, &add_value) == Ordering::Equal {
                                         Value::Single(v)
                                     } else {
-                                        // TODO: recover index name
                                         return Err(IndexChangeError::IndexDuplicateKey(
-                                            "".to_string(),
+                                            index_name.to_string(),
                                             format!("{}", k),
                                         ));
                                     }
@@ -443,8 +425,8 @@ impl IndexTransactionKeeper {
 
     pub fn range<K, V, R>(&self, index: IndexId, range: R) -> Option<IntoIter<K>>
     where
-        K: IndexType,
-        V: IndexType,
+        K: IndexTypeInternal,
+        V: IndexTypeInternal,
         R: RangeBounds<K>,
     {
         self.indexex_changes.get(&index).map(|x| resolve_range(&x.0, range))
@@ -518,17 +500,17 @@ fn map_read_err(r: ReadError) -> GenericError {
     }
 }
 
-impl<'a, K: IndexType, V: IndexType> IndexKeeper<K, V> for IndexSegmentKeeper<'a> {
+impl<'a, K: IndexTypeInternal, V: IndexTypeInternal> IndexKeeper<K, V> for IndexSegmentKeeper<'a> {
     fn load(&self, node: &NodeRef) -> PERes<Node<K, V>> {
         let rec = self
             .store
-            .read_snap_fn(self.segment, node, self.snapshot, |x| deserialize(x))
+            .read_snap_fn(self.segment, node, self.snapshot, deserialize)
             .map_err(map_read_err)?
-            .unwrap()?;
+            .unwrap();
         Ok(rec)
     }
     fn get_root(&self) -> PERes<Option<NodeRef>> {
-        Ok(self.root.clone())
+        Ok(self.root)
     }
     fn value_mode(&self) -> ValueMode {
         self.value_mode.clone()
@@ -544,7 +526,7 @@ struct LockData {
     counter: u32,
 }
 
-pub struct IndexSegmentKeeperTx<'a, K: IndexType, V: IndexType> {
+pub struct IndexSegmentKeeperTx<'a, K, V> {
     name: String,
     index_id: IndexId,
     root: Option<NodeRef>,
@@ -559,7 +541,7 @@ pub struct IndexSegmentKeeperTx<'a, K: IndexType, V: IndexType> {
     updated_root: bool,
 }
 
-impl<'a, K: IndexType, V: IndexType> IndexSegmentKeeperTx<'a, K, V> {
+impl<'a, K: IndexTypeInternal, V: IndexTypeInternal> IndexSegmentKeeperTx<'a, K, V> {
     pub fn new(
         name: &str,
         index_id: &IndexId,
@@ -590,17 +572,17 @@ impl<'a, K: IndexType, V: IndexType> IndexSegmentKeeperTx<'a, K, V> {
         let segment = index_id_to_segment_id_data(&self.index_id);
         if let Some(m) = &self.changed {
             for (node_ref, node) in m {
-                self.store.update(self.tx, segment, node_ref, &serialize(&node.0)?)?;
+                self.store.update(self.tx, segment, node_ref, &serialize(&node.0))?;
             }
         }
         if self.updated_root {
-            Indexes::update_index_root(self.store, self.tx, &self.index_id, self.root.clone())?;
+            Indexes::update_index_root(self.store, self.tx, &self.index_id, self.root)?;
         }
         Ok(())
     }
 }
 
-impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<'a, K, V> {
+impl<'a, K: IndexTypeInternal, V: IndexTypeInternal> IndexModify<K, V> for IndexSegmentKeeperTx<'a, K, V> {
     fn load_modify(&self, node: &NodeRef) -> PIRes<Option<(Rc<Node<K, V>>, u16)>> {
         if let Some(m) = &self.changed {
             if let Some(n) = m.get(node) {
@@ -610,10 +592,10 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         let segment = index_id_to_segment_id_data(&self.index_id);
         if let Some((rec, version)) = self
             .store
-            .read_tx_internal_fn(self.tx, segment, node, |x| deserialize(x))
+            .read_tx_internal_fn(self.tx, segment, node, deserialize)
             .map_err(map_read_err)?
         {
-            Ok(Some((Rc::new(rec?), version)))
+            Ok(Some((Rc::new(rec), version)))
         } else {
             Ok(None)
         }
@@ -629,7 +611,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         } else {
             let segment = index_id_to_segment_id_data(&self.index_id);
             if self.store.lock_record(self.tx, segment, node, version)? {
-                self.locked.insert(node.clone(), LockData { version, counter: 1 });
+                self.locked.insert(*node, LockData { version, counter: 1 });
                 Ok(true)
             } else {
                 Ok(false)
@@ -647,7 +629,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
     }
 
     fn unlock(&mut self, node: &NodeRef) -> PIRes<bool> {
-        if let Entry::Occupied(mut x) = self.locked.entry(node.clone()) {
+        if let Entry::Occupied(mut x) = self.locked.entry(*node) {
             x.get_mut().counter -= 1;
             if x.get().counter == 0 {
                 x.remove();
@@ -668,11 +650,11 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
             self.root = config.get_root();
             self.config_version = version;
         }
-        Ok(self.root.clone())
+        Ok(self.root)
     }
     fn unlock_config(&mut self) -> PIRes<bool> {
         let config_id = Indexes::get_config_id(self.store, self.tx, &self.index_id)?.0;
-        if let Entry::Occupied(mut x) = self.locked.entry(config_id.clone()) {
+        if let Entry::Occupied(mut x) = self.locked.entry(config_id) {
             x.get_mut().counter -= 1;
             if x.get().counter == 0 {
                 x.remove();
@@ -702,7 +684,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
             .lock_record(self.tx, segment, &config_id, self.config_version)?
         {
             self.locked.insert(
-                config_id.clone(),
+                config_id,
                 LockData {
                     version: self.config_version,
                     counter: 1,
@@ -719,12 +701,11 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
 
     fn insert(&mut self, node: Node<K, V>) -> PIRes<NodeRef> {
         let segment = index_id_to_segment_id_data(&self.index_id);
-        let node_ref = self.store.insert_record(self.tx, &segment, &serialize(&node)?)?;
+        let node_ref = self.store.insert_record(self.tx, &segment, &serialize(&node))?;
         self.changed
             .get_or_insert_with(HashMap::new)
-            .insert(node_ref.clone(), (Rc::new(node), 1));
-        self.locked
-            .insert(node_ref.clone(), LockData { version: 1, counter: 1 });
+            .insert(node_ref, (Rc::new(node), 1));
+        self.locked.insert(node_ref, LockData { version: 1, counter: 1 });
         Ok(node_ref)
     }
 
@@ -732,7 +713,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         debug_assert!(self.locked.contains_key(node_ref));
         self.changed
             .get_or_insert_with(HashMap::new)
-            .insert(node_ref.clone(), (Rc::new(node), version));
+            .insert(*node_ref, (Rc::new(node), version));
         Ok(())
     }
 
@@ -759,7 +740,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
     }
 }
 
-impl<'a, K: IndexType, V: IndexType> IndexKeeper<K, V> for IndexSegmentKeeperTx<'a, K, V> {
+impl<'a, K: IndexTypeInternal, V: IndexTypeInternal> IndexKeeper<K, V> for IndexSegmentKeeperTx<'a, K, V> {
     fn load(&self, node: &NodeRef) -> PERes<Node<K, V>> {
         if let Some(m) = &self.changed {
             if let Some(n) = m.get(node) {
@@ -769,13 +750,13 @@ impl<'a, K: IndexType, V: IndexType> IndexKeeper<K, V> for IndexSegmentKeeperTx<
         let segment = index_id_to_segment_id_data(&self.index_id);
         let (rec, _) = self
             .store
-            .read_tx_internal_fn(self.tx, segment, node, |x| deserialize(x))
+            .read_tx_internal_fn(self.tx, segment, node, deserialize)
             .map_err(map_read_err)?
             .unwrap();
-        rec
+        Ok(rec)
     }
     fn get_root(&self) -> PERes<Option<NodeRef>> {
-        Ok(self.root.clone())
+        Ok(self.root)
     }
     fn value_mode(&self) -> ValueMode {
         self.value_mode.clone()
@@ -788,8 +769,9 @@ impl<'a, K: IndexType, V: IndexType> IndexKeeper<K, V> for IndexSegmentKeeperTx<
 
 /// Index Iterator implementation for iterating on a range of keys considering in transaction
 /// changes
-pub struct TxIndexRawIter<K: IndexType, V: IndexType> {
-    index_name: IndexId,
+pub struct TxIndexRawIter<K, V> {
+    index_id: IndexId,
+    index_name: String,
     in_tx: Option<IntoIter<K>>,
     persistent: Option<IndexRawIter<K, V>>,
     in_tx_front: Option<Option<K>>,
@@ -801,16 +783,18 @@ pub struct TxIndexRawIter<K: IndexType, V: IndexType> {
 
 impl<K, V> TxIndexRawIter<K, V>
 where
-    K: IndexType,
-    V: IndexType,
+    K: IndexTypeInternal,
+    V: IndexTypeInternal,
 {
     pub fn new(
-        index_name: IndexId,
+        index_id: IndexId,
+        index_name: String,
         in_tx: Option<IntoIter<K>>,
         persistent: Option<IndexRawIter<K, V>>,
         value_mode: ValueMode,
     ) -> TxIndexRawIter<K, V> {
         TxIndexRawIter {
+            index_id,
             index_name,
             in_tx,
             persistent,
@@ -826,17 +810,21 @@ where
         tx: &mut TransactionImpl,
         vm: ValueMode,
         index: IndexId,
+        index_name: &str,
         k: K,
         pers: Option<Value<V>>,
     ) -> Option<(K, Value<V>)> {
-        tx.apply_changes(vm, index, &k, pers).unwrap_or(None).map(|v| (k, v))
+        tx.apply_changes(vm, index, index_name, &k, pers)
+            .unwrap_or(None)
+            .map(|v| (k, v))
     }
 
     pub fn next(&mut self, persy_impl: &Arc<PersyImpl>, tx: &mut TransactionImpl) -> Option<(K, Value<V>)> {
         loop {
             let vm = self.value_mode.clone();
-            let index = self.index_name.clone();
-            let apply_changes = |k, o| Self::apply_changes(tx, vm, index, k, o);
+            let index = self.index_id.clone();
+            let index_name = &self.index_name;
+            let apply_changes = |k, o| Self::apply_changes(tx, vm, index, index_name, k, o);
             match (&mut self.in_tx, &mut self.persistent) {
                 (Some(it), Some(pers)) => {
                     match (
@@ -896,8 +884,9 @@ where
     pub fn next_back(&mut self, persy_impl: &Arc<PersyImpl>, tx: &mut TransactionImpl) -> Option<(K, Value<V>)> {
         loop {
             let vm = self.value_mode.clone();
-            let index = self.index_name.clone();
-            let apply_changes = |k, o| Self::apply_changes(tx, vm, index, k, o);
+            let index = self.index_id.clone();
+            let index_name = &self.index_name;
+            let apply_changes = |k, o| Self::apply_changes(tx, vm, index, index_name, k, o);
             match (&mut self.in_tx, &mut self.persistent) {
                 (Some(it), Some(pers)) => {
                     match (
@@ -956,7 +945,7 @@ where
     }
 }
 
-pub struct IndexRawIter<K: IndexType, V: IndexType> {
+pub struct IndexRawIter<K, V> {
     index_id: IndexId,
     read_snapshot: SnapshotId,
     iter: PageIter<K, V>,
@@ -966,8 +955,8 @@ pub struct IndexRawIter<K: IndexType, V: IndexType> {
 
 impl<K, V> IndexRawIter<K, V>
 where
-    K: IndexType,
-    V: IndexType,
+    K: IndexTypeInternal,
+    V: IndexTypeInternal,
 {
     pub fn new(
         index_id: IndexId,
@@ -1032,11 +1021,19 @@ where
 
 #[cfg(test)]
 mod tests {
-    use super::{ByteVec, IndexTransactionKeeper, IndexType, ValueChange};
-    use crate::id::{IndexId, PersyId, RecRef, SegmentId};
+    use super::{ByteVec, IndexTransactionKeeper, ValueChange};
+    use crate::{
+        id::{IndexId, PersyId, RecRef, SegmentId},
+        index::{config::IndexTypeInternal, string_wrapper::StringWrapper},
+    };
+
     use std::fmt::Debug;
 
-    fn keeper_test_for_type<K: IndexType + PartialEq, V: IndexType + Debug + PartialEq>(k: K, dk: K, v: V) {
+    fn keeper_test_for_type<K: IndexTypeInternal + PartialEq, V: IndexTypeInternal + Debug + PartialEq>(
+        k: K,
+        dk: K,
+        v: V,
+    ) {
         let name = IndexId::new(SegmentId::new(30), SegmentId::new(40));
         let mut keeper = IndexTransactionKeeper::new();
         keeper.put(name.clone(), k.clone(), v.clone());
@@ -1061,7 +1058,11 @@ mod tests {
         keeper_test_for_type::<i128, i128>(10, 15, 10);
         keeper_test_for_type::<f32, f32>(20.0, 10.0, 20.0);
         keeper_test_for_type::<f64, f64>(20.0, 10.0, 20.0);
-        keeper_test_for_type::<String, String>("a".to_string(), "b".to_string(), "a".to_string());
+        keeper_test_for_type::<StringWrapper, StringWrapper>(
+            StringWrapper::new("a".to_string()),
+            StringWrapper::new("b".to_string()),
+            StringWrapper::new("a".to_string()),
+        );
         keeper_test_for_type::<ByteVec, ByteVec>(vec![0, 1].into(), vec![0, 2].into(), vec![0, 1].into());
         let id = PersyId(RecRef::new(10, 20));
         let id1 = PersyId(RecRef::new(20, 20));
