@@ -2,7 +2,7 @@ use crate::{
     error::{IndexChangeError, PIRes},
     id::RecRef,
     index::{
-        config::{IndexOrd, IndexType, ValueMode},
+        config::{IndexOrd, IndexTypeInternal, ValueMode},
         tree::PosRef,
     },
 };
@@ -106,7 +106,7 @@ impl<K> Nodes<K> {
 impl<K: IndexOrd + Clone> Nodes<K> {
     pub fn new_from_split(left: NodeRef, values: &[(K, NodeRef)]) -> Nodes<K> {
         let keys = values.iter().map(|z| z.0.clone()).collect();
-        let mut pointers: Vec<NodeRef> = values.iter().map(|z| z.1.clone()).collect();
+        let mut pointers: Vec<NodeRef> = values.iter().map(|z| z.1).collect();
         pointers.insert(0, left);
         Nodes {
             keys,
@@ -124,18 +124,18 @@ impl<K: IndexOrd + Clone> Nodes<K> {
     pub fn find(&self, k: &K) -> PosRef<K> {
         match self.keys.binary_search_by(|x| compare(x, k)) {
             Ok(index) => {
-                let sibling = Some(self.pointers[index].clone());
-                PosRef::new(k, index + 1, self.pointers[index + 1].clone(), sibling)
+                let sibling = Some(self.pointers[index]);
+                PosRef::new(k, index + 1, self.pointers[index + 1], sibling)
             }
             Err(index) => {
                 let sibling = if index > 0 {
-                    Some(self.pointers[index - 1].clone())
+                    Some(self.pointers[index - 1])
                 } else if self.pointers.len() > index + 1 {
-                    Some(self.pointers[index + 1].clone())
+                    Some(self.pointers[index + 1])
                 } else {
                     None
                 };
-                PosRef::new(k, index, self.pointers[index].clone(), sibling)
+                PosRef::new(k, index, self.pointers[index], sibling)
             }
         }
     }
@@ -163,13 +163,13 @@ impl<K: IndexOrd + Clone> Nodes<K> {
     }
 
     pub fn get(&self, pos: usize) -> NodeRef {
-        self.pointers[pos].clone()
+        self.pointers[pos]
     }
 
     pub fn insert_after(&mut self, pos: usize, values: &mut Vec<(K, NodeRef)>) {
         values.reverse();
         for (key, node) in values.iter() {
-            self.add(pos, key, node.clone());
+            self.add(pos, key, *node);
         }
     }
 
@@ -420,7 +420,7 @@ impl<K: IndexOrd + Clone, V: Clone> Leaf<K, V> {
     }
 }
 
-impl<K: IndexType, V: IndexOrd + Clone> Leaf<K, V> {
+impl<K: IndexTypeInternal, V: IndexOrd + Clone> Leaf<K, V> {
     pub fn insert_or_update(&mut self, k: &K, v: &V, value_mode: ValueMode, index_name: &str) -> PIRes<()> {
         match self.entries.binary_search_by(|n| compare(&n.key, k)) {
             Ok(index) => {
