@@ -1,13 +1,13 @@
 mod helpers;
 use helpers::create_and_drop;
-use persy::Persy;
-use persy::{ByteVec, IndexIter, IndexType, PRes, TxIndexIter, Value, ValueMode};
+use persy::{ByteVec, IndexIter, IndexType, TxIndexIter, ValueMode, PE};
+use persy::{Persy, PersyError};
 
 fn create_and_drop_index<F>(test: &str, f: F)
 where
     F: FnOnce(&Persy, &str),
 {
-    create_and_drop_index_mode(test, ValueMode::CLUSTER, f);
+    create_and_drop_index_mode(test, ValueMode::Cluster, f);
 }
 
 fn create_and_drop_index_mode<F>(test: &str, mode: ValueMode, f: F)
@@ -34,7 +34,7 @@ where
 fn test_create_drop_index() {
     create_and_drop("create_drop_index", |persy| {
         let mut tx = persy.begin().expect("begin transaction works");
-        tx.create_index::<u8, u8>("index1", ValueMode::CLUSTER)
+        tx.create_index::<u8, u8>("index1", ValueMode::Cluster)
             .expect("index created correctly");
         assert!(tx.exists_index("index1").expect("exists wokrs"));
         let prep = tx.prepare().expect("prepare with index works");
@@ -54,14 +54,14 @@ fn test_create_drop_index() {
 fn test_create_put_remove_get_drop_index_same_tx() {
     create_and_drop("create_drop_index", |persy| {
         let mut tx = persy.begin().expect("begin transaction works");
-        tx.create_index::<u8, u8>("index1", ValueMode::CLUSTER)
+        tx.create_index::<u8, u8>("index1", ValueMode::Cluster)
             .expect("index created correctly");
         tx.put::<u8, u8>("index1", 10, 12).expect("put works correctly");
-        let res = tx.get::<u8, u8>("index1", &10).expect("get works correctly");
-        assert_eq!(Some(Value::SINGLE(12)), res);
+        let mut res = tx.get::<u8, u8>("index1", &10).expect("get works correctly");
+        assert_eq!(Some(12), res.next());
         tx.remove::<u8, u8>("index1", 10, None).expect("put works correctly");
         let res = tx.get::<u8, u8>("index1", &10).expect("get works correctly");
-        assert_eq!(None, res);
+        assert_eq!(0, res.len());
 
         let prep = tx.prepare().expect("prepare with index works");
         prep.commit().expect("commit with index works");
@@ -72,29 +72,39 @@ fn test_create_put_remove_get_drop_index_same_tx() {
     });
 }
 
-fn test_for_type<K: IndexType, V>(persy: &Persy, name: &str, k: K, v: V) -> PRes<()>
+#[derive(Debug)]
+struct TErr {
+    pe: PersyError,
+}
+impl<T: Into<PersyError>> From<PE<T>> for TErr {
+    fn from(e: PE<T>) -> Self {
+        TErr { pe: e.error().into() }
+    }
+}
+
+fn test_for_type<K: IndexType, V>(persy: &Persy, name: &str, k: K, v: V) -> Result<(), TErr>
 where
     V: IndexType + std::fmt::Debug + PartialEq,
 {
     let mut tx = persy.begin()?;
-    tx.create_index::<K, V>(name, ValueMode::CLUSTER)?;
+    tx.create_index::<K, V>(name, ValueMode::Cluster)?;
     tx.put::<K, V>(name, k.clone(), v.clone())?;
-    let res = tx.get::<K, V>(name, &k)?;
-    assert_eq!(Some(Value::SINGLE(v.clone())), res);
+    let mut res = tx.get::<K, V>(name, &k)?;
+    assert_eq!(Some(v.clone()), res.next());
     let prep = tx.prepare()?;
     prep.commit()?;
 
-    let res = persy.get::<K, V>(name, &k)?;
-    assert_eq!(Some(Value::SINGLE(v)), res);
+    let mut res = persy.get::<K, V>(name, &k)?;
+    assert_eq!(Some(v), res.next());
     let mut tx = persy.begin()?;
     tx.remove::<K, V>(name, k.clone(), None)?;
     let res = tx.get::<K, V>(name, &k)?;
-    assert!(res.is_none());
+    assert!(res.len() == 0);
     let prep = tx.prepare()?;
     prep.commit()?;
 
     let res = persy.get::<K, V>(name, &k)?;
-    assert!(res.is_none());
+    assert!(res.len() == 0);
     let mut tx = persy.begin()?;
     tx.drop_index(name)?;
     let prep = tx.prepare()?;
@@ -132,36 +142,36 @@ fn test_put_get_index() {
         let prep = tx.prepare().expect("prepare with index works");
         prep.commit().expect("commit with index works");
 
-        let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::SINGLE(12)));
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
     });
 }
 
 #[test]
 fn test_put_get_tx_index() {
-    create_and_drop_index("create_drop_index", |persy, index_name| {
+    create_and_drop_index("create_put_get_tx_index", |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
-        let res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::SINGLE(12)));
+        let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
         let prep = tx.prepare().expect("prepare with index works");
         prep.commit().expect("commit with index works");
 
-        let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::SINGLE(12)));
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
     });
 }
 
 #[test]
 fn test_put_remove_index() {
-    create_and_drop_index("create_drop_index", |persy, index_name| {
+    create_and_drop_index("create_put_remove_index", |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
         let prep = tx.prepare().expect("prepare with index works");
         prep.commit().expect("commit with index works");
 
-        let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::SINGLE(12)));
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
 
         let mut tx = persy.begin().expect("begin transaction works");
         tx.remove::<u8, u8>(index_name, 10, None).expect("put works correctly");
@@ -169,13 +179,13 @@ fn test_put_remove_index() {
         prep.commit().expect("commit with index works");
 
         let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, None);
+        assert_eq!(res.len(), 0);
     });
 }
 
 #[test]
 fn test_duplicate_put() {
-    create_and_drop_index_mode("create_drop_index", ValueMode::EXCLUSIVE, |persy, index_name| {
+    create_and_drop_index_mode("duplicate_put", ValueMode::Exclusive, |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
         tx.put::<u8, u8>(index_name, 10, 20).expect("put works correctly");
@@ -185,21 +195,21 @@ fn test_duplicate_put() {
 
 #[test]
 fn test_same_value_no_duplicate_put() {
-    create_and_drop_index_mode("create_drop_index", ValueMode::EXCLUSIVE, |persy, index_name| {
+    create_and_drop_index_mode("same_value_no_duplicate", ValueMode::Exclusive, |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
         let prep = tx.prepare().expect("prepare with index works");
         prep.commit().expect("commit with index works");
 
-        let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::SINGLE(12)));
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
     });
 }
 
 #[test]
 fn test_duplicate_second_put() {
-    create_and_drop_index_mode("create_drop_index", ValueMode::EXCLUSIVE, |persy, index_name| {
+    create_and_drop_index_mode("create_drop_index", ValueMode::Exclusive, |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
 
@@ -214,7 +224,7 @@ fn test_duplicate_second_put() {
 
 #[test]
 fn test_duplicate_tx_put() {
-    create_and_drop_index_mode("create_drop_index", ValueMode::EXCLUSIVE, |persy, index_name| {
+    create_and_drop_index_mode("duplicate_put_tx", ValueMode::Exclusive, |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
         tx.put::<u8, u8>(index_name, 10, 20).expect("put works correctly");
@@ -224,17 +234,39 @@ fn test_duplicate_tx_put() {
 
 #[test]
 fn test_same_value_no_duplicate_put_tx() {
-    create_and_drop_index_mode("create_drop_index", ValueMode::EXCLUSIVE, |persy, index_name| {
+    create_and_drop_index_mode(
+        "some_value_no_duplicate_put_tx",
+        ValueMode::Exclusive,
+        |persy, index_name| {
+            let mut tx = persy.begin().expect("begin transaction works");
+            tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+            let mut value = tx.get::<u8, u8>(index_name, &10).expect("get works");
+            assert_eq!(value.next(), Some(12));
+        },
+    );
+}
+
+#[test]
+fn test_put_one_index() {
+    create_and_drop_index("put_one_index", |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
-        tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
-        assert_eq!(tx.get::<u8, u8>(index_name, &10).ok(), Some(Some(Value::SINGLE(12))));
+        let res = tx.one::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res, Some(12));
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+        let snap = persy.snapshot().expect("snapshot ok");
+        let res = snap.one::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res, Some(12));
+        let res = persy.one::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res, Some(12));
     });
 }
 
 #[test]
 fn test_put_remove_index_one_tx() {
-    create_and_drop_index("create_drop_index", |persy, index_name| {
+    create_and_drop_index("put_remove_index_one", |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
         tx.remove::<u8, u8>(index_name, 10, None).expect("put works correctly");
@@ -242,13 +274,13 @@ fn test_put_remove_index_one_tx() {
         prep.commit().expect("commit with index works");
 
         let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, None);
+        assert_eq!(res.len(), 0);
     });
 }
 
 #[test]
-fn test_mupltiple_values_put_get() {
-    create_and_drop_index("create_drop_index", |persy, index_name| {
+fn test_multiple_values_put_get() {
+    create_and_drop_index("multiple_values_put", |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
 
@@ -257,25 +289,25 @@ fn test_mupltiple_values_put_get() {
         prep.commit().expect("commit with index works");
 
         let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::CLUSTER(vec![12, 14])));
+        assert_eq!(res.collect::<Vec<_>>(), vec![12, 14]);
     });
 }
 
 #[test]
-fn test_mupltiple_values_put_get_tx() {
-    create_and_drop_index("create_drop_index", |persy, index_name| {
+fn test_multiple_values_put_get_tx() {
+    create_and_drop_index("multiple_values_put_get_tx", |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
 
         tx.put::<u8, u8>(index_name, 10, 14).expect("put works correctly");
         let res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::CLUSTER(vec![12, 14])));
+        assert_eq!(res.collect::<Vec<_>>(), vec![12, 14]);
     });
 }
 
 #[test]
-fn test_mupltiple_put_same_value_get() {
-    create_and_drop_index("create_drop_index", |persy, index_name| {
+fn test_multiple_put_same_value_get() {
+    create_and_drop_index("multiple_put same_value_gut", |persy, index_name| {
         let mut tx = persy.begin().expect("begin transaction works");
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
 
@@ -283,11 +315,178 @@ fn test_mupltiple_put_same_value_get() {
         let prep = tx.prepare().expect("prepare with index works");
         prep.commit().expect("commit with index works");
 
-        let res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::SINGLE(12)));
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
     });
 }
 
+#[test]
+fn test_multiple_put_same_value_intervalled_get() {
+    create_and_drop_index("multiple_put same_value_intervalled_get", |persy, index_name| {
+        let mut tx = persy.begin().expect("begin transaction works");
+        tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut tx = persy.begin().expect("begin transaction works");
+        tx.put::<u8, u8>(index_name, 10, 13).expect("put works correctly");
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut tx = persy.begin().expect("begin transaction works");
+        tx.put::<u8, u8>(index_name, 10, 14).expect("put works correctly");
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut tx = persy.begin().expect("begin transaction works");
+        tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+        let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
+        assert_eq!(res.next(), Some(13));
+        assert_eq!(res.next(), Some(14));
+        assert_eq!(res.next(), None);
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
+        assert_eq!(res.next(), Some(13));
+        assert_eq!(res.next(), Some(14));
+        assert_eq!(res.next(), None);
+    });
+}
+
+#[test]
+fn test_multiple_put_remove_same_value_intervalled_get() {
+    create_and_drop_index("multiple_put remove_same_value_intervalled_get", |persy, index_name| {
+        let mut tx = persy.begin().expect("begin transaction works");
+        tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+        tx.put::<u8, u8>(index_name, 10, 13).expect("put works correctly");
+        tx.put::<u8, u8>(index_name, 10, 14).expect("put works correctly");
+        tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
+        assert_eq!(res.next(), Some(13));
+        assert_eq!(res.next(), Some(14));
+        assert_eq!(res.next(), None);
+
+        let mut tx = persy.begin().expect("begin transaction works");
+        tx.remove::<u8, u8>(index_name, 10, Some(12))
+            .expect("put works correctly");
+        tx.remove::<u8, u8>(index_name, 10, Some(11))
+            .expect("put works correctly");
+        tx.remove::<u8, u8>(index_name, 10, Some(13))
+            .expect("put works correctly");
+        tx.remove::<u8, u8>(index_name, 10, Some(12))
+            .expect("put works correctly");
+        tx.remove::<u8, u8>(index_name, 10, Some(13))
+            .expect("put works correctly");
+        let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(14));
+        assert_eq!(res.next(), None);
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(14));
+        assert_eq!(res.next(), None);
+    });
+}
+#[test]
+fn test_multiple_put_same_value_intervalled_get_same_tx() {
+    create_and_drop_index(
+        "multiple_put same_value_intervalled_get_same_tx",
+        |persy, index_name| {
+            let mut tx = persy.begin().expect("begin transaction works");
+            tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 13).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 14).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+
+            let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+            assert_eq!(res.next(), Some(12));
+            assert_eq!(res.next(), Some(13));
+            assert_eq!(res.next(), Some(14));
+            assert_eq!(res.next(), None);
+            let prep = tx.prepare().expect("prepare with index works");
+            prep.commit().expect("commit with index works");
+
+            let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+            assert_eq!(res.next(), Some(12));
+            assert_eq!(res.next(), Some(13));
+            assert_eq!(res.next(), Some(14));
+            assert_eq!(res.next(), None);
+        },
+    );
+}
+
+#[test]
+fn test_multiple_put_same_value_inverted_order_get_same_tx() {
+    create_and_drop_index("multiple_put same_value_inverted_get_same_tx", |persy, index_name| {
+        let mut tx = persy.begin().expect("begin transaction works");
+        tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+        tx.put::<u8, u8>(index_name, 10, 11).expect("put works correctly");
+        tx.put::<u8, u8>(index_name, 10, 13).expect("put works correctly");
+        tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+        tx.put::<u8, u8>(index_name, 10, 13).expect("put works correctly");
+
+        let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(11));
+        assert_eq!(res.next(), Some(12));
+        assert_eq!(res.next(), Some(13));
+        assert_eq!(res.next(), None);
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(11));
+        assert_eq!(res.next(), Some(12));
+        assert_eq!(res.next(), Some(13));
+        assert_eq!(res.next(), None);
+    });
+}
+
+#[test]
+fn test_multiple_put_remove_same_value_inverted_order_get_same_tx() {
+    create_and_drop_index(
+        "multiple_put_remove_same_value_inverted_order_get_same_tx",
+        |persy, index_name| {
+            let mut tx = persy.begin().expect("begin transaction works");
+            tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 11).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 13).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
+            tx.put::<u8, u8>(index_name, 10, 13).expect("put works correctly");
+
+            let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+            assert_eq!(res.next(), Some(11));
+            assert_eq!(res.next(), Some(12));
+            assert_eq!(res.next(), Some(13));
+            assert_eq!(res.next(), None);
+            tx.remove::<u8, u8>(index_name, 10, Some(12))
+                .expect("put works correctly");
+            tx.remove::<u8, u8>(index_name, 10, Some(11))
+                .expect("put works correctly");
+            tx.remove::<u8, u8>(index_name, 10, Some(13))
+                .expect("put works correctly");
+            tx.remove::<u8, u8>(index_name, 10, Some(12))
+                .expect("put works correctly");
+            tx.remove::<u8, u8>(index_name, 10, Some(13))
+                .expect("put works correctly");
+            let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+            assert_eq!(res.next(), None);
+            let prep = tx.prepare().expect("prepare with index works");
+            prep.commit().expect("commit with index works");
+
+            let mut res = persy.get::<u8, u8>(index_name, &10).expect("get works correctly");
+            assert_eq!(res.next(), None);
+        },
+    );
+}
+
 #[test]
 fn test_index_browse() {
     create_and_drop_index("test_index_browse", |persy, index_name| {
@@ -305,9 +504,9 @@ fn test_index_browse() {
         let mut iter: IndexIter<u8, u8> = persy.range(index_name, ..).expect("browse works correctly");
         assert_eq!(iter.next().unwrap().0, start);
         let mut last = None;
-        for x in iter {
+        for mut x in iter {
             last = Some(x.0.clone());
-            assert_eq!(x.1, Value::SINGLE(12));
+            assert_eq!(x.1.next(), Some(12));
             count -= 1;
         }
         assert_eq!(Some(to), last);
@@ -332,9 +531,9 @@ fn test_index_range() {
         let mut iter: IndexIter<u8, u8> = persy.range(index_name, start..=to).expect("range works correctly");
         assert_eq!(iter.next().unwrap().0, start);
         let mut last = None;
-        for x in iter {
+        for mut x in iter {
             last = Some(x.0.clone());
-            assert_eq!(x.1, Value::SINGLE(12));
+            assert_eq!(x.1.next(), Some(12));
             count -= 1;
         }
         assert_eq!(Some(to), last);
@@ -360,9 +559,9 @@ fn test_index_range_rev() {
         let mut iter = base.rev();
         assert_eq!(iter.next().unwrap().0, to);
         let mut last = None;
-        for x in iter {
+        for mut x in iter {
             last = Some(x.0.clone());
-            assert_eq!(x.1, Value::SINGLE(12));
+            assert_eq!(x.1.next(), Some(12));
             count -= 1;
         }
         assert_eq!(Some(start), last);
@@ -384,9 +583,9 @@ fn test_index_browse_tx() {
         {
             let mut iter: TxIndexIter<u8, u8> = tx.range(index_name, ..).expect("browse works correctly");
             assert_eq!(iter.next().unwrap().0, start);
-            for x in iter {
+            for mut x in iter {
                 last = Some(x.0.clone());
-                assert_eq!(x.1, Value::SINGLE(12));
+                assert_eq!(x.1.next(), Some(12));
                 count -= 1;
             }
         }
@@ -413,9 +612,9 @@ fn test_index_range_tx() {
             let mut iter: TxIndexIter<u8, u8> = tx.range(index_name, start..=to).expect("range works correctly");
             assert_eq!(iter.next().unwrap().0, start);
             let mut last = None;
-            for x in iter {
+            for mut x in iter {
                 last = Some(x.0.clone());
-                assert_eq!(x.1, Value::SINGLE(12));
+                assert_eq!(x.1.next(), Some(12));
                 assert!(x.0 != 20u8);
                 count -= 1;
             }
@@ -445,9 +644,9 @@ fn test_index_range_tx_access() {
             assert_eq!(iter.next().unwrap().0, start);
             iter.tx().insert("seg", "data".as_bytes()).expect("insert works");
             let mut last = None;
-            while let Some(x) = iter.next_tx() {
+            while let Some(mut x) = iter.next_tx() {
                 last = Some(x.0.clone());
-                assert_eq!(x.1, Value::SINGLE(12));
+                assert_eq!(x.1.next(), Some(12));
                 x.2.insert("seg", "data".as_bytes()).expect("insert works");
                 count -= 1;
             }
@@ -477,9 +676,9 @@ fn test_index_range_tx_rev() {
             let mut iter = base.rev();
             assert_eq!(iter.next().unwrap().0, to);
             let mut last = None;
-            for x in iter {
+            for mut x in iter {
                 last = Some(x.0.clone());
-                assert_eq!(x.1, Value::SINGLE(12));
+                assert_eq!(x.1.next(), Some(12));
                 assert!(x.0 != 20u8);
                 count -= 1;
             }
@@ -511,9 +710,9 @@ fn test_index_range_tx_change_committed() {
             let mut iter: TxIndexIter<u8, u8> = tx.range(index_name, start..=to).expect("range works correctly");
             assert_eq!(iter.next().unwrap().0, start);
             let mut last = None;
-            for x in iter {
+            for mut x in iter {
                 last = Some(x.0.clone());
-                assert_eq!(x.1, Value::SINGLE(12));
+                assert_eq!(x.1.next(), Some(12));
                 assert!(x.0 != 20u8);
                 count -= 1;
             }
@@ -526,9 +725,9 @@ fn test_index_range_tx_change_committed() {
             let mut iter = base.rev();
             assert_eq!(iter.next().unwrap().0, to);
             let mut last = None;
-            for x in iter {
+            for mut x in iter {
                 last = Some(x.0.clone());
-                assert_eq!(x.1, Value::SINGLE(12));
+                assert_eq!(x.1.next(), Some(12));
                 assert!(x.0 != 20u8);
                 count -= 1;
             }
@@ -548,8 +747,8 @@ fn test_mupltiple_put_same_value_get_tx() {
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
 
         tx.put::<u8, u8>(index_name, 10, 12).expect("put works correctly");
-        let res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
-        assert_eq!(res, Some(Value::SINGLE(12)));
+        let mut res = tx.get::<u8, u8>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
     });
 }
 
@@ -557,9 +756,9 @@ fn test_mupltiple_put_same_value_get_tx() {
 pub fn test_list_indexes() {
     create_and_drop("test_list_indexes", |persy| {
         let mut tx = persy.begin().expect("error on transaction begin");
-        tx.create_index::<u8, u8>("one", ValueMode::REPLACE)
+        tx.create_index::<u8, u8>("one", ValueMode::Replace)
             .expect("error on index creation");
-        tx.create_index::<u32, u32>("two", ValueMode::REPLACE)
+        tx.create_index::<u32, u32>("two", ValueMode::Replace)
             .expect("error on index creation");
         let fin = tx.prepare().expect("error on commit prepare");
         fin.commit().expect("error on commit");
@@ -577,14 +776,14 @@ pub fn test_list_indexes() {
 pub fn test_list_indexes_tx() {
     create_and_drop("test_list_indexes", |persy| {
         let mut tx = persy.begin().expect("error on transaction begin");
-        tx.create_index::<u8, u8>("one", ValueMode::REPLACE)
+        tx.create_index::<u8, u8>("one", ValueMode::Replace)
             .expect("error on index creation");
-        tx.create_index::<u32, u32>("two", ValueMode::REPLACE)
+        tx.create_index::<u32, u32>("two", ValueMode::Replace)
             .expect("error on index creation");
         let fin = tx.prepare().expect("error on commit prepare");
         fin.commit().expect("error on commit");
         let mut tx = persy.begin().expect("error on transaction begin");
-        tx.create_index::<u8, u8>("three", ValueMode::REPLACE)
+        tx.create_index::<u8, u8>("three", ValueMode::Replace)
             .expect("error on index creation");
         tx.drop_index("two").expect("error on index creation");
 
@@ -598,3 +797,58 @@ pub fn test_list_indexes_tx() {
         assert_eq!(segments.len(), 0);
     });
 }
+
+#[test]
+fn test_grow_and_shrink() {
+    create_and_drop("grow_and_shrink", |persy| {
+        let mut tx = persy.begin().expect("begin transaction works");
+        let index_name = "grow_and_shrink";
+        tx.create_index::<i32, i32>(index_name, ValueMode::Cluster).unwrap();
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+        let mut tx = persy.begin().expect("begin transaction works");
+        for v in 0..1000 {
+            tx.put::<i32, i32>(index_name, v, 12).expect("put works correctly");
+            tx.put::<i32, i32>(index_name, v, 13).expect("put works correctly");
+            tx.put::<i32, i32>(index_name, v, 14).expect("put works correctly");
+            if v % 100 == 0 {
+                let prep = tx.prepare().expect("prepare with index works");
+                prep.commit().expect("commit with index works");
+                tx = persy.begin().expect("begin transaction works");
+            }
+        }
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut res = persy.get::<i32, i32>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), Some(12));
+        assert_eq!(res.next(), Some(13));
+        assert_eq!(res.next(), Some(14));
+        assert_eq!(res.next(), None);
+        let count = persy
+            .range::<i32, i32, _>(index_name, ..)
+            .expect("get works correctly")
+            .count();
+        assert_eq!(count, 1000);
+
+        let mut tx = persy.begin().expect("begin transaction works");
+        for v in 0..1000 {
+            tx.remove::<i32, i32>(index_name, v, None).expect("put works correctly");
+            if v % 100 == 0 {
+                let prep = tx.prepare().expect("prepare with index works");
+                prep.commit().expect("commit with index works");
+                tx = persy.begin().expect("begin transaction works");
+            }
+        }
+        let prep = tx.prepare().expect("prepare with index works");
+        prep.commit().expect("commit with index works");
+
+        let mut res = persy.get::<i32, i32>(index_name, &10).expect("get works correctly");
+        assert_eq!(res.next(), None);
+        let count = persy
+            .range::<i32, i32, _>(index_name, ..)
+            .expect("get works correctly")
+            .count();
+        assert_eq!(count, 0);
+    });
+}
